#!/usr/bin/python
'''test client for point_cluster_grasp_planner.  Requires tabletop_node.launch to be running (in tabletop_object_detector).  Add Markers of topic 'grasp_markers' to see the poses being tested'''

from __future__ import division
import roslib
roslib.load_manifest('pr2_pick_and_place_demos')
import rospy
from object_manipulation_msgs.srv import FindClusterBoundingBox, FindClusterBoundingBoxRequest
from tabletop_object_detector.srv import TabletopDetection, TabletopDetectionRequest
from object_manipulation_msgs.srv import GraspPlanning, GraspPlanningRequest
from object_manipulation_msgs.msg import GraspPlanningErrorCode
from visualization_msgs.msg import Marker
import tf.transformations
import numpy
from geometry_msgs.msg import Point
import scipy
import time
import object_manipulator.draw_functions as draw_functions
from object_manipulator.convert_functions import *


#call the tabletop object detector to get the table and clusters in the scene
def call_object_detector():

    req = TabletopDetectionRequest()
    req.return_clusters = 1
    req.return_models = 0
    service_name = "/object_detection"
    rospy.loginfo("waiting for object_detection service")
    rospy.wait_for_service(service_name)
    serv = rospy.ServiceProxy(service_name, TabletopDetection)
    try:
        res = serv(req)
    except rospy.ServiceException, e:
        rospy.logerr("error when calling object_detection: %s"%e)  
        return 0
    return (res.detection.table, res.detection.clusters)


#call plan_point_cluster_grasp to get candidate grasps for a cluster
def call_plan_point_cluster_grasp(cluster):
    
    req = GraspPlanningRequest()
    req.target.cluster = cluster
    req.target.type = 2
    req.arm_name = "right_arm"
    req.collision_support_surface_name = "table"
    service_name = "plan_point_cluster_grasp"
    rospy.loginfo("waiting for plan_point_cluster_grasp service")
    rospy.wait_for_service(service_name)
    rospy.loginfo("service found")
    serv = rospy.ServiceProxy(service_name, GraspPlanning)
    try:
        res = serv(req)
    except rospy.ServiceException, e:
        rospy.logerr("error when calling plan_point_cluster_grasp: %s"%e)  
        return 0
    if res.error_code.value != 0:
        return []
    
    grasp_poses = [grasp.grasp_pose for grasp in res.grasps]
    return grasp_poses


#call find_cluster_bounding_box to get the bounding box for a cluster
def call_find_cluster_bounding_box(cluster):
    
    req = FindClusterBoundingBoxRequest()
    req.cluster = cluster
    service_name = "find_cluster_bounding_box"
    rospy.loginfo("waiting for find_cluster_bounding_box service")
    rospy.wait_for_service(service_name)
    rospy.loginfo("service found")
    serv = rospy.ServiceProxy(service_name, FindClusterBoundingBox)
    try:
        res = serv(req)
    except rospy.ServiceException, e:
        rospy.logerr("error when calling find_cluster_bounding_box: %s"%e)  
        return 0
    if not res.error_code:
        return (res.pose, res.box_dims)
    else:
        return (None, None)


if __name__ == "__main__":
    rospy.init_node('test_point_cluster_grasp_planner', anonymous=True)
    tf_broadcaster = tf.TransformBroadcaster()
    tf_listener = tf.TransformListener()
    draw_functions = draw_functions.DrawFunctions('grasp_markers')

    (table, clusters) = call_object_detector()
    print "table:\n", table

    while(1):
        for cluster in clusters:
            draw_functions.clear_grasps(num=1000)
            time.sleep(.05)

            #points = point_cloud_to_mat(cluster)
            #draw_functions.draw_rviz_points(points)

            #get the bounding box and draw it in blue
            (box_pose, box_dims) = call_find_cluster_bounding_box(cluster)
            if box_pose == None:
                continue
            box_mat = pose_to_mat(box_pose.pose)
            box_ranges = [[-box_dims.x/2, -box_dims.y/2, -box_dims.z/2],
                          [box_dims.x/2, box_dims.y/2, box_dims.z/2]]

            draw_functions.draw_rviz_box(box_mat, box_ranges, 'base_link', ns = 'bounding box', color = [0,0,1], opaque = 0.25, duration = 60)
            print "box_mat:\n", ppmat(box_mat)
            print "box_ranges:", box_ranges
            
            print "Run the blue cluster? Enter y to run, enter to go to the next, q to exit"
            c = raw_input()

            if c == 'y':
                #make up for missing TF messages in data
                if not tf_listener.canTransform('base_link', 'narrow_stereo_optical_frame', rospy.Duration(1.0)):
                    pos = [0.123, 0.936, 1.094]
                    quat = [0.678, -0.670, 0.193, 0.231]
                    tf_broadcaster.sendTransform(pos, quat, rospy.Time.now(), "base_link", "narrow_stereo_optical_frame") 

                draw_functions.clear_rviz_points('grasp_markers')
                grasps = call_plan_point_cluster_grasp(cluster)

                print "number of grasps returned:", len(grasps)
                print "drawing grasps, press enter to continue"
                draw_functions.draw_grasps(grasps, cluster.header.frame_id, pause = 1)
                raw_input()

                draw_functions.clear_rviz_points('grasp_markers')
                draw_functions.clear_grasps(num = 200)
                time.sleep(.05)

            elif c == 'q':
                break
        else:
            continue
        break

